@@ -1,24 +1,21 @@
-use std::convert::TryInto;
 use std::fs::File;
 use std::path::Path;
-use std::sync::Arc;
 
 use clap::{App, Arg};
-use parquet::file::reader::{FileReader, Length, SerializedFileReader};
+use parquet::file::reader::{FileReader, SerializedFileReader};
 use parquet::record::reader::RowIter;
 use rusoto_core::Region;
-use rusoto_s3::{ListObjectsV2Output, ListObjectsV2Request, S3Client, S3};
 use url::Url;
 
 mod http_reader;
 use http_reader::HttpChunkReader;
-mod buzz;
-use buzz::{s3, CachedFile, RangeCache};
+mod s3_reader;
+use s3_reader::S3ChunkReader;
 
 enum Source {
     File(String),
     Http(String),
-    S3(String, usize),
+    S3(String),
 }
 
 fn output_rows(iter: RowIter, offset: u32, limit: i32) {
@@ -56,40 +53,20 @@ async fn print_json_from(source: Source, offset: u32, limit: i32) {
             });
             blocking_task.await.unwrap();
         }
-        Source::S3(url_str, concurrency) => {
+        Source::S3(url_str) => {
             let url = Url::parse(&url_str).unwrap();
             let host_str = url.host_str().unwrap();
             let key = &url.path()[1..];
 
-            let s3_client = S3Client::new(Region::default());
-            let list_req = ListObjectsV2Request {
-                bucket: String::from(host_str),
-                prefix: Some(String::from(key)),
-                ..Default::default()
-            };
-            let list_res: ListObjectsV2Output = s3_client.list_objects_v2(list_req).await.unwrap();
-            let object_found = &list_res.contents.unwrap()[0];
-            let size = object_found.size.unwrap().try_into().unwrap();
-
-            let is_parallel = concurrency > 1;
-            let cache = RangeCache::new(concurrency).await;
-            let (dler_id, dler_creator) = s3::downloader_creator(Region::default().name());
-            let file_id = s3::file_id(host_str, key);
-            let file = CachedFile::new(
-                file_id,
-                size,
-                Arc::new(cache),
-                dler_id,
-                dler_creator,
-                is_parallel,
-            );
-
-            let pre_size: u64 = if is_parallel { 1024 * 1024 } else { size - 4 };
-            file.prefetch(file.len() - pre_size, pre_size as usize);
-
-            let file_reader = SerializedFileReader::new(file).unwrap();
+            let mut reader = S3ChunkReader::new_unknown_size(
+                (String::from(host_str), String::from(key)),
+                Region::default(),
+            )
+            .await;
+            reader.start(Region::default()).await;
 
             let blocking_task = tokio::task::spawn_blocking(move || {
+                let file_reader = SerializedFileReader::new(reader).unwrap();
                 output_rows(file_reader.get_row_iter(None).unwrap(), offset, limit);
             });
             blocking_task.await.unwrap();
@@ -124,23 +101,14 @@ async fn main() {
                 .about("Maximum number of rows to output")
                 .takes_value(true),
         )
-        .arg(
-            Arg::new("concurrency")
-                .short('c')
-                .long("concurrency")
-                .value_name("NUMBER")
-                .about("Maximum number of concurrent column downloads (S3 only, default: 10)")
-                .takes_value(true),
-        )
         .get_matches();
 
     let offset: u32 = matches.value_of_t("offset").unwrap_or(0);
     let limit: i32 = matches.value_of_t("limit").unwrap_or(-1);
     let file: String = matches.value_of_t("FILE").unwrap_or_else(|e| e.exit());
-    let concurrency: usize = matches.value_of_t("concurrency").unwrap_or(10);
 
     if file.as_str().starts_with("s3://") {
-        print_json_from(Source::S3(file, concurrency), offset, limit).await;
+        print_json_from(Source::S3(file), offset, limit).await;
     } else if file.as_str().starts_with("http") {
         print_json_from(Source::Http(file), offset, limit).await;
     } else {
